@@ -91,13 +91,15 @@ aggregate.sf = function(x, by, FUN, ..., do_union = TRUE, simplify = TRUE,
 #' Areal-weighted interpolation of polygon data
 #'
 #' Areal-weighted interpolation of polygon data
+#' @name interpolate_aw
 #' @param x object of class \code{sf}, for which we want to aggregate attributes
 #' @param to object of class \code{sf} or \code{sfc}, with the target geometries
 #' @param extensive logical; if TRUE, the attribute variables are assumed to be spatially extensive (like population) and the sum is preserved, otherwise, spatially intensive (like population density) and the mean is preserved.
 #' @param ... ignored
+#' @param keep_NA logical; if \code{TRUE}, return all features in \code{to}, if \code{FALSE} return only those with non-NA values (but with \code{row.names} the index corresponding to the feature in \code{to})
 #' @examples
 #' nc = st_read(system.file("shape/nc.shp", package="sf"))
-#' g = st_make_grid(nc, n = c(20,10))
+#' g = st_make_grid(nc, n = c(10, 5))
 #' a1 = st_interpolate_aw(nc["BIR74"], g, extensive = FALSE)
 #' sum(a1$BIR74) / sum(nc$BIR74) # not close to one: property is assumed spatially intensive
 #' a2 = st_interpolate_aw(nc["BIR74"], g, extensive = TRUE)
@@ -105,12 +107,13 @@ aggregate.sf = function(x, by, FUN, ..., do_union = TRUE, simplify = TRUE,
 #' sum(a2$BIR74) / sum(nc$BIR74)
 #' a1$intensive = a1$BIR74
 #' a1$extensive = a2$BIR74
-#' plot(a1[c("intensive", "extensive")], key.pos = 4)
+#' \donttest{plot(a1[c("intensive", "extensive")], key.pos = 4)}
 #' @export
 st_interpolate_aw = function(x, to, extensive, ...) UseMethod("st_interpolate_aw")
 
 #' @export
-st_interpolate_aw.sf = function(x, to, extensive, ...) {
+#' @name interpolate_aw
+st_interpolate_aw.sf = function(x, to, extensive, ..., keep_NA = FALSE) {
 	if (!inherits(to, "sf") && !inherits(to, "sfc")) {
 		to <- try(st_as_sf(to))
 		if (inherits(to, "try-error"))
@@ -119,7 +122,7 @@ st_interpolate_aw.sf = function(x, to, extensive, ...) {
 
 	if (! all_constant(x))
 		warning("st_interpolate_aw assumes attributes are constant or uniform over areas of x")
-	i = st_intersection(st_geometry(x), st_geometry(to))
+	i = st_intersection(st_geometry(x), st_geometry(to), dimensions = "polygon")
 	idx = attr(i, "idx")
 
 	# https://stackoverflow.com/questions/57767022/how-do-you-use-st-interpolate-aw-with-polygon-layers-that-legitimately-include-p
@@ -129,7 +132,11 @@ st_interpolate_aw.sf = function(x, to, extensive, ...) {
 	i[two_d] = st_cast(i[two_d], "MULTIPOLYGON")
 
 	x_st = st_set_geometry(x, NULL)[idx[,1],, drop=FALSE]   # create st table, remove geom
-	area_i = st_area(i)
+	if (any(!sapply(x_st, is.numeric)))
+		stop("x contains non-numeric column(s)")
+	area_i = try(st_area(i), silent = TRUE)
+	if (inherits(area_i, "try-error"))
+		area_i <- st_area(st_make_valid(i)) # work-around for https://github.com/r-spatial/sf/issues/1810
 	x_st$...area_st = unclass(area_i)
 
 	x_st = if (extensive) {
@@ -144,7 +151,12 @@ st_interpolate_aw.sf = function(x, to, extensive, ...) {
 			lapply(x_st, function(v) v * x_st$...area_st / x_st$...area_t)
 		}
 	x_st = aggregate(x_st, list(idx[,2]), sum)
-	df = st_sf(x_st, geometry = st_geometry(to)[x_st$Group.1])
+	df = if (keep_NA) {
+			ix = rep(NA_integer_, length(to))
+			ix[x_st$Group.1] = seq_along(x_st$Group.1)
+			st_sf(x_st[ix,], geometry = st_geometry(to))
+		} else
+			st_sf(x_st, geometry = st_geometry(to)[x_st$Group.1], row.names = x_st$Group.1)
 	# clean up:
 	df$...area_t = df$...area_st = df$...area_s = df$Group.1 = NULL
 	st_set_agr(df, "aggregate")
