@@ -8,6 +8,11 @@
 #'
 #' If \code{e1} is of class \code{sfc}, and \code{e2} is a length 2 numeric, then it is considered a two-dimensional point (and if needed repeated as such) only for operations \code{+} and \code{-}, in other cases the individual numbers are repeated; see commented examples.
 #'
+#' @details
+#' It has been reported (https://github.com/r-spatial/sf/issues/2067) that
+#' certain ATLAS versions result in invalid polygons, where the final point
+#' in a ring is no longer equal to the first point. In that case, setting
+#' the precisions with \link{st_set_precision} may help.
 #' @return object of class \code{sfg}
 #' @export
 #'
@@ -31,7 +36,7 @@
 #' a = st_buffer(st_point(c(0,0)), 2)
 #' b = a + c(2, 0)
 #' p = function(m) { plot(c(a,b)); plot(eval(parse(text=m)), col=grey(.9), add = TRUE); title(m) }
-#' lapply(c('a | b', 'a / b', 'a & b', 'a %/% b'), p)
+#' o = lapply(c('a | b', 'a / b', 'a & b', 'a %/% b'), p)
 #' par(opar)
 #' sfc = st_sfc(st_point(0:1), st_point(2:3))
 #' sfc + c(2,3) # added to EACH geometry
@@ -42,7 +47,7 @@ Ops.sfg <- function(e1, e2) {
 		switch (.Generic,
 			"-" = return(e1 * -1.0),
 			"+" = return(e1),
-			stop(paste("unary", .Generic, "not defined for \"sfg\" objects"))
+			stop(paste("unary", .Generic, 'not defined for "sfg" objects'))
 		)
 	}
 
