@@ -9,19 +9,24 @@
 #' @param dist numeric; buffer distance for all, or for each of the elements in \code{x}; in case
 #' \code{dist} is a \code{units} object, it should be convertible to \code{arc_degree} if
 #' \code{x} has geographic coordinates, and to \code{st_crs(x)$units} otherwise
-#' @param nQuadSegs integer; number of segments per quadrant (fourth of a circle), for all or per-feature
-#' @param endCapStyle character; style of line ends, one of 'ROUND', 'FLAT', 'SQUARE'
-#' @param joinStyle character; style of line joins, one of 'ROUND', 'MITRE', 'BEVEL'
-#' @param mitreLimit numeric; limit of extension for a join if \code{joinStyle} 'MITRE' is used (default 1.0, minimum 0.0)
+#' @param nQuadSegs integer; number of segments per quadrant (fourth of a circle), for all or per-feature; see details
+#' @param endCapStyle character; style of line ends, one of 'ROUND', 'FLAT', 'SQUARE'; see details
+#' @param joinStyle character; style of line joins, one of 'ROUND', 'MITRE', 'BEVEL'; see details
+#' @param mitreLimit numeric; limit of extension for a join if \code{joinStyle} 'MITRE' is used (default 1.0, minimum 0.0); see details
 #' @param singleSide logical; if \code{TRUE}, single-sided buffers are returned for linear geometries,
-#' in which case negative \code{dist} values give buffers on the right-hand side, positive on the left.
+#' in which case negative \code{dist} values give buffers on the right-hand side, positive on the left; see details
 #' @param ... passed on to \code{s2_buffer_cells}
 #' @return an object of the same class of \code{x}, with manipulated geometry.
 #' @export
 #' @details \code{st_buffer} computes a buffer around this geometry/each geometry. If any of \code{endCapStyle},
 #' \code{joinStyle}, or \code{mitreLimit} are set to non-default values ('ROUND', 'ROUND', 1.0 respectively) then
 #' the underlying 'buffer with style' GEOS function is used.
+#' If a negative buffer returns empty polygons instead of shrinking, set st_use_s2() to FALSE
 #' See \href{https://postgis.net/docs/ST_Buffer.html}{postgis.net/docs/ST_Buffer.html} for details.
+#' 
+#' \code{nQuadSegs}, \code{endCapsStyle}, \code{joinStyle}, \code{mitreLimit} and \code{singleSide} only
+#' work when the GEOS back-end is used: for projected coordinates or when \code{sf_use_s2()} is set
+#' to \code{FALSE}.
 #' @examples
 #'
 #' ## st_buffer, style options (taken from rgeos gBuffer)
@@ -80,7 +85,7 @@ st_buffer.sfg = function(x, dist, nQuadSegs = 30,
 	if (is.na(mitreLimit) || !mitreLimit > 0) stop("mitreLimit must be > 0")
 	if (is.na(ecs)) stop("endCapStyle must be 'ROUND', 'FLAT', or 'SQUARE'")
 	if (is.na(js))  stop("joinStyle must be 'ROUND', 'MITRE', or 'BEVEL'")
-	if (any(is.na(singleSide))) stop("singleSide should be TRUE or FALSE")
+	if (anyNA(singleSide)) stop("singleSide should be TRUE or FALSE")
 	styls$with_styles = TRUE
 	styls$endCapStyle = ecs
 	styls$joinStyle = js
@@ -95,9 +100,7 @@ st_buffer.sfc = function(x, dist, nQuadSegs = 30,
 	if (longlat && sf_use_s2()) {
 #		if (!missing(nQuadSegs) || !missing(endCapStyle) || !missing(joinStyle) ||
 #				!missing(mitreLimit) || !missing(singleSide))
-#			warning("all bufer style parameters are ignored; set st_use_s2(FALSE) first to use them")
-		if (! requireNamespace("s2", quietly = TRUE))
-			stop("package s2 required, please install it first")
+#			warning("all buffer style parameters are ignored; set st_use_s2(FALSE) first to use them")
 		if (inherits(dist, "units")) {
 			if (!inherits(try(units(dist) <- as_units("rad"), silent = TRUE), "try-error"))
 				return(st_as_sfc(s2::s2_buffer_cells(x, dist, radius = 1, ...),
@@ -118,7 +121,8 @@ st_buffer.sfc = function(x, dist, nQuadSegs = 30,
 				stop("x does not have a crs set: can't convert units")
 			if (is.null(st_crs(x)$units))
 				stop("x has a crs without units: can't convert units")
-			units(dist) = crs_parameters(st_crs(x))$ud_unit
+			if (!is.null(st_crs(x)$ud_unit))
+				units(dist) = st_crs(x)$ud_unit
 		}
 		dist = rep(dist, length.out = length(x))
 		nQ = rep(nQuadSegs, length.out = length(x))
@@ -166,17 +170,21 @@ st_boundary.sfc = function(x)
 
 #' @export
 st_boundary.sf = function(x) {
-	st_geometry(x) <- st_boundary(st_geometry(x))
-	x
+	st_set_geometry(x, st_boundary(st_geometry(x)))
 }
 
 #' @name geos_unary
 #' @export
 #' @details \code{st_convex_hull} creates the convex hull of a set of points
+#' @seealso \link[grDevices]{chull} for a more efficient algorithm for calculating the convex hull
 #' @examples
 #' nc = st_read(system.file("shape/nc.shp", package="sf"))
-#' plot(st_convex_hull(nc))
-#' plot(nc, border = grey(.5))
+#' nc_g = st_geometry(nc)
+#' plot(st_convex_hull(nc_g))
+#' plot(nc_g, border = grey(.5), add = TRUE)
+#' pt = st_combine(st_sfc(st_point(c(0,80)), st_point(c(120,80)), st_point(c(240,80))))
+#' st_convex_hull(pt) # R2
+#' st_convex_hull(st_set_crs(pt, 'OGC:CRS84')) # S2
 st_convex_hull = function(x)
 	UseMethod("st_convex_hull")
 
@@ -185,37 +193,100 @@ st_convex_hull.sfg = function(x)
 	get_first_sfg(st_convex_hull(st_sfc(x)))
 
 #' @export
-st_convex_hull.sfc = function(x)
-	st_sfc(CPL_geos_op("convex_hull", x, numeric(0), integer(0), numeric(0), logical(0)))
+st_convex_hull.sfc = function(x) {
+	if (isTRUE(st_is_longlat(x)) && sf_use_s2())
+		st_as_sfc(s2::s2_convex_hull(x), crs = st_crs(x))
+	else
+		st_sfc(CPL_geos_op("convex_hull", x, numeric(0), integer(0), numeric(0), logical(0)))
+}
 
 #' @export
 st_convex_hull.sf = function(x) {
-	st_geometry(x) <- st_convex_hull(st_geometry(x))
-	x
+	st_set_geometry(x, st_convex_hull(st_geometry(x)))
 }
 
 #' @name geos_unary
 #' @export
-#' @details \code{st_simplify} simplifies lines by removing vertices
-#' @param preserveTopology logical; carry out topology preserving simplification? May be specified for each, or for all feature geometries. Note that topology is preserved only for single feature geometries, not for sets of them.
-#' @param dTolerance numeric; tolerance parameter, specified for all or for each feature geometry.
-st_simplify = function(x, preserveTopology = FALSE, dTolerance = 0.0)
+#' @details \code{st_concave_hull} creates the concave hull of a geometry
+#' @param ratio numeric; fraction convex: 1 returns the convex hulls, 0 maximally concave hulls
+#' @param allow_holes logical; if \code{TRUE}, the resulting concave hull may have holes
+#' @examples
+#' set.seed(131)
+#' if (compareVersion(sf_extSoftVersion()[["GEOS"]], "3.11.0") > -1) {
+#'  pts = cbind(runif(100), runif(100))
+#'  m = st_multipoint(pts)
+#'  co = sf:::st_concave_hull(m, 0.3)
+#'  coh = sf:::st_concave_hull(m, 0.3, allow_holes = TRUE)
+#'  plot(co, col = 'grey')
+#'  plot(coh, add = TRUE, border = 'red')
+#'  plot(m, add = TRUE)
+#' }
+st_concave_hull = function(x, ratio, ..., allow_holes)
+	UseMethod("st_concave_hull")
+
+#' @export
+st_concave_hull.sfg = function(x, ratio, ..., allow_holes = FALSE)
+	get_first_sfg(st_concave_hull(st_sfc(x), ratio, allow_holes))
+
+#' @export
+st_concave_hull.sfc = function(x, ratio, ..., allow_holes = FALSE) {
+	stopifnot(!missing(ratio), ratio >= 0, ratio <= 1.0, is.logical(allow_holes), !is.na(allow_holes))
+	st_sfc(CPL_geos_op("concave_hull", x, ratio, integer(0), numeric(0), allow_holes))
+}
+
+#' @export
+st_concave_hull.sf = function(x, ratio, ..., allow_holes = FALSE) {
+	st_set_geometry(x, st_concave_hull(st_geometry(x), ratio, allow_holes))
+}
+
+#' @name geos_unary
+#' @export
+#' @details \code{st_simplify} simplifies lines by removing vertices. 
+#' @param preserveTopology logical; carry out topology preserving
+#'   simplification? May be specified for each, or for all feature geometries.
+#'   Note that topology is preserved only for single feature geometries, not for
+#'   sets of them. If not specified (i.e. the default), then it is internally
+#'   set equal to \code{FALSE} when the input data is specified with projected
+#'   coordinates or \code{sf_use_s2()} returns \code{FALSE}. Ignored in all the
+#'   other cases (with a warning when set equal to \code{FALSE}) since the
+#'   function implicitly calls \code{s2::s2_simplify} which always preserve
+#'   topological relationships (per single feature).
+#' @param dTolerance numeric; tolerance parameter, specified for all or for each
+#'   feature geometry. If you run \code{st_simplify}, the input data is
+#'   specified with long-lat coordinates and \code{sf_use_s2()} returns
+#'   \code{TRUE}, then the value of \code{dTolerance} must be specified in
+#'   meters.
+#' @examples
+#'
+#' # st_simplify examples:
+#' op = par(mfrow = c(2, 3), mar = rep(0, 4))
+#' plot(nc_g[1])
+#' plot(st_simplify(nc_g[1], dTolerance = 1e3)) # 1000m
+#' plot(st_simplify(nc_g[1], dTolerance = 5e3)) # 5000m
+#' nc_g_planar = st_transform(nc_g, 2264) # planar coordinates, US foot
+#' plot(nc_g_planar[1])
+#' plot(st_simplify(nc_g_planar[1], dTolerance = 1e3)) # 1000 foot
+#' plot(st_simplify(nc_g_planar[1], dTolerance = 5e3)) # 5000 foot
+#' par(op)
+#'
+st_simplify = function(x, preserveTopology, dTolerance = 0.0)
 	UseMethod("st_simplify")
 
 #' @export
-st_simplify.sfg = function(x, preserveTopology = FALSE, dTolerance = 0.0)
+st_simplify.sfg = function(x, preserveTopology, dTolerance = 0.0)
 	get_first_sfg(st_simplify(st_sfc(x), preserveTopology, dTolerance = dTolerance))
 
 #' @export
-st_simplify.sfc = function(x, preserveTopology = FALSE, dTolerance = 0.0) {
+st_simplify.sfc = function(x, preserveTopology, dTolerance = 0.0) {
 	ll = isTRUE(st_is_longlat(x))
 	if (ll && sf_use_s2()) {
-		if (!missing(preserveTopology))
-			warning("argument preserveTopology is ignored")
-		if (! requireNamespace("s2", quietly = TRUE))
-			stop("package s2 required, please install it first")
+		if (!missing(preserveTopology) && isFALSE(preserveTopology))
+			warning("argument preserveTopology cannot be set to FALSE when working with ellipsoidal coordinates since the algorithm behind st_simplify always preserves topological relationships")
 		st_as_sfc(s2::s2_simplify(x, dTolerance), crs = st_crs(x))
 	} else {
+		if (missing(preserveTopology)) {
+			preserveTopology = FALSE
+		}
 		stopifnot(mode(preserveTopology) == 'logical')
 		if (ll)
 			warning("st_simplify does not correctly simplify longitude/latitude data, dTolerance needs to be in decimal degrees")
@@ -227,9 +298,8 @@ st_simplify.sfc = function(x, preserveTopology = FALSE, dTolerance = 0.0) {
 }
 
 #' @export
-st_simplify.sf = function(x, preserveTopology = FALSE, dTolerance = 0.0) {
-	st_geometry(x) <- st_simplify(st_geometry(x), preserveTopology, dTolerance)
-	x
+st_simplify.sf = function(x, preserveTopology, dTolerance = 0.0) {
+	st_set_geometry(x, st_simplify(st_geometry(x), preserveTopology, dTolerance))
 }
 
 #' @name geos_unary
@@ -245,7 +315,7 @@ st_triangulate.sfg = function(x, dTolerance = 0.0, bOnlyEdges = FALSE)
 
 #' @export
 st_triangulate.sfc = function(x, dTolerance = 0.0, bOnlyEdges = FALSE) {
-	if (CPL_geos_version() >= "3.4.0") {
+	if (compareVersion(CPL_geos_version(), "3.4.0") > -1) { # >= ; see https://github.com/r-spatial/sf/issues/1653
 		if (isTRUE(st_is_longlat(x)))
 			warning("st_triangulate does not correctly triangulate longitude/latitude data")
 		st_sfc(CPL_geos_op("triangulate", x, numeric(0), integer(0),
@@ -257,10 +327,122 @@ st_triangulate.sfc = function(x, dTolerance = 0.0, bOnlyEdges = FALSE) {
 
 #' @export
 st_triangulate.sf = function(x, dTolerance = 0.0, bOnlyEdges = FALSE) {
-	st_geometry(x) <- st_triangulate(st_geometry(x), dTolerance, bOnlyEdges)
-	x
+	st_set_geometry(x, st_triangulate(st_geometry(x), dTolerance, bOnlyEdges))
 }
 
+#' @name geos_unary
+#' @export
+#' @details \code{st_triangulate_constrained} returns the constrained delaunay triangulation of polygons; requires GEOS version 3.10 or above
+#' @examples
+#' if (compareVersion(sf_extSoftVersion()[["GEOS"]], "3.10.0") > -1) {
+#'  pts = rbind(c(0,0), c(1,0), c(1,1), c(.5,.5), c(0,1), c(0,0))
+#'  po = st_polygon(list(pts))
+#'  co = st_triangulate_constrained(po)
+#'  tr = st_triangulate(po)
+#'  plot(po, col = NA, border = 'grey', lwd = 15)
+#'  plot(tr, border = 'green', col = NA, lwd = 5, add = TRUE)
+#'  plot(co, border = 'red', col = 'NA', add = TRUE)
+#' }
+st_triangulate_constrained = function(x)
+	UseMethod("st_triangulate_constrained")
+
+#' @export
+st_triangulate_constrained.sfg = function(x)
+	get_first_sfg(st_triangulate_constrained(st_sfc(x)))
+
+#' @export
+st_triangulate_constrained.sfc = function(x) {
+	if (compareVersion(CPL_geos_version(), "3.10.0") > -1) { # >= ; see https://github.com/r-spatial/sf/issues/1653
+		if (isTRUE(st_is_longlat(x)))
+			warning("st_triangulate does not correctly triangulate longitude/latitude data")
+		st_sfc(CPL_geos_op("triangulate_constrained", x, numeric(0), integer(0), numeric(0), logical(0)))
+	} else
+		stop("for triangulate_constrained, GEOS version 3.10.0 or higher is required")
+}
+
+#' @export
+st_triangulate_constrained.sf = function(x) {
+	st_set_geometry(x, st_triangulate_constrained(st_geometry(x)))
+}
+
+#' @name geos_unary
+#' @export
+#' @details \code{st_inscribed_circle} returns the maximum inscribed circle for polygon geometries. 
+#' For \code{st_inscribed_circle}, if \code{nQuadSegs} is 0 a 2-point LINESTRING is returned with the
+#' center point and a boundary point of every circle, otherwise a circle (buffer) is returned where
+#' \code{nQuadSegs} controls the number of points per quadrant to approximate the circle.
+#' \code{st_inscribed_circle} requires GEOS version 3.9 or above
+#' @examples
+#' if (compareVersion(sf_extSoftVersion()[["GEOS"]], "3.9.0") > -1) {
+#'   nc_t = st_transform(nc, 'EPSG:2264')
+#'   x = st_inscribed_circle(st_geometry(nc_t))
+#'   plot(st_geometry(nc_t), asp = 1, col = grey(.9))
+#'   plot(x, add = TRUE, col = '#ff9999')
+#' }
+st_inscribed_circle = function(x, dTolerance, ...)
+	UseMethod("st_inscribed_circle")
+
+#' @export
+st_inscribed_circle.sfg = function(x, dTolerance, ...) {
+	get_first_sfg(st_inscribed_circle(st_sfc(x), dTolerance, ...))
+}
+
+#' @export
+st_inscribed_circle.sfc = function(x, dTolerance = sqrt(st_area(st_set_crs(x, NA_crs_)))/1000, ..., nQuadSegs = 30) {
+	if (compareVersion(CPL_geos_version(), "3.9.0") > -1) { # >=
+		if (isTRUE(st_is_longlat(x)))
+			warning("st_inscribed_circle does not work correctly for longitude/latitude data")
+		nQ = rep(nQuadSegs, length.out = length(x))
+		ret = st_sfc(CPL_geos_op("inscribed_circle", x, nQ, integer(0),
+			dTolerance = rep(as.double(dTolerance), length.out = length(x)), logical(0),
+			bOnlyEdges = as.integer(FALSE)))
+		if (any(nQuadSegs > 0)) {
+			pts = st_cast(ret, "POINT")
+			idx = seq(1, length(pts) * 2, by = 2)
+			ret = st_buffer(pts[idx], st_length(st_set_crs(ret, NA_crs_)), nQuadSegs = nQuadSegs)
+		} 
+		ret
+	} else
+		stop("for st_inscribed_circle, GEOS version 3.9.0 or higher is required")
+}
+
+#' @export
+st_inscribed_circle.sf = function(x, dTolerance, ...) {
+	st_set_geometry(x, st_inscribed_circle(st_geometry(x), dTolerance), ...)
+}
+
+#' @name geos_unary
+#' @details \code{st_minimum_rotated_rectangle} returns the minimum
+#' rotated rectangular POLYGON which encloses the input geometry. The
+#' rectangle has width equal to the minimum diameter, and a longer
+#' length. If the convex hill of the input is degenerate (a line or
+#' point) a linestring or point is returned.
+#' @export
+st_minimum_rotated_rectangle = function(x, ...)
+	UseMethod("st_minimum_rotated_rectangle")
+
+#' @export
+st_minimum_rotated_rectangle.sfg = function(x, ...) {
+	get_first_sfg(st_minimum_rotated_rectangle(st_sfc(x), ...))
+}
+
+#' @export
+st_minimum_rotated_rectangle.sfc = function(x, ...) {
+	if (compareVersion(CPL_geos_version(), "3.9.0") > -1) { # >=
+		if (isTRUE(st_is_longlat(x)))
+			warning("st_minimum_rotated_rectangle does not work correctly for longitude/latitude data")
+		st_sfc(CPL_geos_op("minimum_rotated_rectangle", x, 0L, integer(0),
+			dTolerance = 0., logical(0), bOnlyEdges = as.integer(FALSE)))
+	} else
+		stop("for st_minimum_rotated_rectangle, GEOS version 3.9.0 or higher is required")
+}
+
+#' @export
+st_minimum_rotated_rectangle.sf = function(x, dTolerance, ...) {
+	st_set_geometry(x, st_minimum_rotated_rectangle(st_geometry(x)), ...)
+}
+
+
 #' @name geos_unary
 #' @export
 #' @param envelope object of class \code{sfc} or \code{sfg} containing a \code{POLYGON} with the envelope for a voronoi diagram; this only takes effect when it is larger than the default envelope, chosen when \code{envelope} is an empty polygon
@@ -269,7 +451,7 @@ st_triangulate.sf = function(x, dTolerance = 0.0, bOnlyEdges = FALSE) {
 #' set.seed(1)
 #' x = st_multipoint(matrix(runif(10),,2))
 #' box = st_polygon(list(rbind(c(0,0),c(1,0),c(1,1),c(0,1),c(0,0))))
-#' if (sf_extSoftVersion()["GEOS"] >= "3.5.0") {
+#' if (compareVersion(sf_extSoftVersion()[["GEOS"]], "3.5.0") > -1) {
 #'  v = st_sfc(st_voronoi(x, st_sfc(box)))
 #'  plot(v, col = 0, border = 1, axes = TRUE)
 #'  plot(box, add = TRUE, col = 0, border = 1) # a larger box is returned, as documented
@@ -288,6 +470,7 @@ st_triangulate.sf = function(x, dTolerance = 0.0, bOnlyEdges = FALSE) {
 #'  plot(pts["id"], pch = 16) # ID is color
 #'  plot(st_set_geometry(pts, "pols")["id"], xlim = c(0,1), ylim = c(0,1), reset = FALSE)
 #'  plot(st_geometry(pts), add = TRUE)
+#'  layout(matrix(1)) # reset plot layout
 #' }
 st_voronoi = function(x, envelope, dTolerance = 0.0, bOnlyEdges = FALSE)
 	UseMethod("st_voronoi")
@@ -298,7 +481,7 @@ st_voronoi.sfg = function(x, envelope = st_polygon(), dTolerance = 0.0, bOnlyEdg
 
 #' @export
 st_voronoi.sfc = function(x, envelope = st_polygon(), dTolerance = 0.0, bOnlyEdges = FALSE) {
-	if (sf_extSoftVersion()["GEOS"] >= "3.5.0") {
+	if (compareVersion(CPL_geos_version(), "3.5.0") > -1) {
 		if (isTRUE(st_is_longlat(x)))
 			warning("st_voronoi does not correctly triangulate longitude/latitude data")
 		st_sfc(CPL_geos_voronoi(x, st_sfc(envelope), dTolerance = dTolerance,
@@ -309,8 +492,7 @@ st_voronoi.sfc = function(x, envelope = st_polygon(), dTolerance = 0.0, bOnlyEdg
 
 #' @export
 st_voronoi.sf = function(x, envelope = st_polygon(), dTolerance = 0.0, bOnlyEdges = FALSE) {
-	st_geometry(x) <- st_voronoi(st_geometry(x), st_sfc(envelope), dTolerance, bOnlyEdges)
-	x
+	st_set_geometry(x, st_voronoi(st_geometry(x), st_sfc(envelope), dTolerance, bOnlyEdges))
 }
 
 #' @name geos_unary
@@ -334,33 +516,35 @@ st_polygonize.sfc = function(x) {
 
 #' @export
 st_polygonize.sf = function(x) {
-	st_geometry(x) <- st_polygonize(st_geometry(x))
-	x
+	st_set_geometry(x, st_polygonize(st_geometry(x)))
 }
 
 #' @name geos_unary
 #' @export
+#' @param directed logical; if \code{TRUE}, lines with opposite directions will not be merged
 #' @details \code{st_line_merge} merges lines. In case of \code{st_line_merge}, \code{x} must be an object of class \code{MULTILINESTRING}, or an \code{sfc} geometry list-column object containing these
 #' @examples
 #' mls = st_multilinestring(list(rbind(c(0,0), c(1,1)), rbind(c(2,0), c(1,1))))
 #' st_line_merge(st_sfc(mls))
-st_line_merge = function(x)
+st_line_merge = function(x, ..., directed = FALSE)
 	UseMethod("st_line_merge")
 
 #' @export
-st_line_merge.sfg = function(x)
-	get_first_sfg(st_line_merge(st_sfc(x)))
+st_line_merge.sfg = function(x, ..., directed = FALSE)
+	get_first_sfg(st_line_merge(st_sfc(x), directed = directed, ...))
 
 #' @export
-st_line_merge.sfc = function(x) {
+st_line_merge.sfc = function(x, ..., directed = FALSE) {
 	stopifnot(inherits(x, "sfc_MULTILINESTRING"))
-	st_sfc(CPL_geos_op("linemerge", x, numeric(0), integer(0), numeric(0), logical(0)))
+	if (directed)
+		st_sfc(CPL_geos_op("linemergedirected", x, numeric(0), integer(0), numeric(0), logical(0)))
+	else
+		st_sfc(CPL_geos_op("linemerge", x, numeric(0), integer(0), numeric(0), logical(0)))
 }
 
 #' @export
-st_line_merge.sf = function(x) {
-	st_geometry(x) <- st_line_merge(st_geometry(x))
-	x
+st_line_merge.sf = function(x, ..., directed = FALSE) {
+	st_set_geometry(x, st_line_merge(st_geometry(x), directed = directed, ...))
 }
 
 #' @name geos_unary
@@ -368,8 +552,8 @@ st_line_merge.sf = function(x) {
 #' @export
 #' @details \code{st_centroid} gives the centroid of a geometry
 #' @examples
-#' plot(nc, axes = TRUE)
-#' plot(st_centroid(nc), add = TRUE, pch = 3)
+#' plot(nc_g, axes = TRUE)
+#' plot(st_centroid(nc_g), add = TRUE, pch = 3, col = 'red')
 #' mp = st_combine(st_buffer(st_sfc(lapply(1:3, function(x) st_point(c(x,x)))), 0.2 * 1:3))
 #' plot(mp)
 #' plot(st_centroid(mp), add = TRUE, col = 'red') # centroid of combined geometry
@@ -399,11 +583,9 @@ st_centroid.sfc = function(x, ..., of_largest_polygon = FALSE) {
 			x[multi] = largest_ring(x[multi])
 	}
 	longlat = isTRUE(st_is_longlat(x))
-	if (longlat && sf_use_s2()) {
-		if (! requireNamespace("s2", quietly = TRUE))
-			stop("package s2 required, please install it first")
+	if (longlat && sf_use_s2())
 		st_as_sfc(s2::s2_centroid(x), crs = st_crs(x))
-	} else { 
+	else { 
 		if (longlat)
 			warning("st_centroid does not give correct centroids for longitude/latitude data")
 		st_sfc(CPL_geos_op("centroid", x, numeric(0), integer(0), numeric(0), logical(0)))
@@ -412,8 +594,8 @@ st_centroid.sfc = function(x, ..., of_largest_polygon = FALSE) {
 
 #' @export
 st_centroid.sf = function(x, ..., of_largest_polygon = FALSE) {
-	if (! all_constant(x))
-		warning("st_centroid assumes attributes are constant over geometries of x")
+	if (any(st_dimension(x) > 0) && !all_constant(x))
+		warning("st_centroid assumes attributes are constant over geometries", call. = FALSE)
 	ret = st_set_geometry(x,
 		st_centroid(st_geometry(x), of_largest_polygon = of_largest_polygon))
 	agr = st_agr(ret)
@@ -426,8 +608,8 @@ st_centroid.sf = function(x, ..., of_largest_polygon = FALSE) {
 #' @export
 #' @details \code{st_point_on_surface} returns a point guaranteed to be on the (multi)surface.
 #' @examples
-#' plot(nc, axes = TRUE)
-#' plot(st_point_on_surface(nc), add = TRUE, pch = 3)
+#' plot(nc_g, axes = TRUE)
+#' plot(st_point_on_surface(nc_g), add = TRUE, pch = 3, col = 'red')
 st_point_on_surface = function(x)
 	UseMethod("st_point_on_surface")
 
@@ -444,8 +626,8 @@ st_point_on_surface.sfc = function(x) {
 
 #' @export
 st_point_on_surface.sf = function(x) {
-	if (! all_constant(x))
-		warning("st_point_on_surface assumes attributes are constant over geometries of x")
+	if (any(st_dimension(x) > 0) && !all_constant(x))
+		warning("st_point_on_surface assumes attributes are constant over geometries", call. = FALSE)
 	st_set_geometry(x, st_point_on_surface(st_geometry(x)))
 }
 
@@ -453,7 +635,7 @@ st_point_on_surface.sf = function(x) {
 #' @export
 #' @details \code{st_reverse} reverses the nodes in a line
 #' @examples
-#' if (sf_extSoftVersion()["GEOS"] >= "3.7.0") {
+#' if (compareVersion(sf_extSoftVersion()[["GEOS"]], "3.7.0") > -1) {
 #'   st_reverse(st_linestring(rbind(c(1,1), c(2,2), c(3,3))))
 #' }
 #nocov start
@@ -510,9 +692,11 @@ st_node.sf = function(x) {
 #' @param ... ignored
 #' @examples
 #' sf = st_sf(a=1, geom=st_sfc(st_linestring(rbind(c(0,0),c(1,1)))), crs = 4326)
-#' seg = st_segmentize(sf, units::set_units(100, km))
-#' seg = st_segmentize(sf, units::set_units(0.01, rad))
-#' nrow(seg$geom[[1]])
+#' if (require(lwgeom, quietly = TRUE)) {
+#'  seg = st_segmentize(sf, units::set_units(100, km))
+#'  seg = st_segmentize(sf, units::set_units(0.01, rad))
+#'  nrow(seg$geom[[1]])
+#' }
 st_segmentize	= function(x, dfMaxLength, ...)
 	UseMethod("st_segmentize")
 
@@ -530,15 +714,14 @@ st_segmentize.sfc	= function(x, dfMaxLength, ...) {
 		lwgeom::st_geod_segmentize(x, dfMaxLength) # takes care of rad or degree units
 	} else {
 		if (! is.na(st_crs(x)) && inherits(dfMaxLength, "units"))
-			units(dfMaxLength) = units(crs_parameters(st_crs(x))$SemiMajor) # might convert
+			units(dfMaxLength) = units(st_crs(x)$SemiMajor) # might convert
 		st_sfc(CPL_gdal_segmentize(x, dfMaxLength), crs = st_crs(x))
 	}
 }
 
 #' @export
 st_segmentize.sf = function(x, dfMaxLength, ...) {
-	st_geometry(x) <- st_segmentize(st_geometry(x), dfMaxLength, ...)
-	x
+	st_set_geometry(x, st_segmentize(st_geometry(x), dfMaxLength, ...))
 }
 
 #' Combine or union feature geometries
@@ -585,20 +768,19 @@ geos_op2_df = function(x, y, geoms) {
 
 # after checking identical crs,
 # call geos_op2 function op on x and y:
-geos_op2_geom = function(op, x, y, s2_model = "closed", ...) {
+# DE-9IM compliant should use model = "closed", more robust seems:
+geos_op2_geom = function(op, x, y, model = "semi-open", ...) {
 	stopifnot(st_crs(x) == st_crs(y))
 	x = st_geometry(x)
 	y = st_geometry(y)
 	longlat = isTRUE(st_is_longlat(x))
 	if (longlat && sf_use_s2()) {
-		if (! requireNamespace("s2", quietly = TRUE))
-			stop("package s2 required, please install it first")
 		fn = switch(op, intersection = s2::s2_intersection,
 				difference = s2::s2_difference,
 				sym_difference = s2::s2_sym_difference,
 				union = s2::s2_union, stop("invalid operator"))
 		# to be optimized -- this doesn't index on y:
-		lst = structure(unlist(lapply(x, fn, y, s2::s2_options(model = s2_model, ...)),
+		lst = structure(unlist(lapply(y, function(yy) fn(x, yy, s2::s2_options(model = model, ...))),
 			recursive = FALSE), class = "s2_geography")
 		e = s2::s2_is_empty(lst)
 		idx = cbind(rep(seq_along(x), length(y)), rep(seq_along(y), each = length(x)))
@@ -629,11 +811,13 @@ get_first_sfg = function(x) {
 #' @export
 #' @return The intersection, difference or symmetric difference between two sets of geometries.
 #' The returned object has the same class as that of the first argument (\code{x}) with the non-empty geometries resulting from applying the operation to all geometry pairs in \code{x} and \code{y}. In case \code{x} is of class \code{sf}, the matching attributes of the original object(s) are added. The \code{sfc} geometry list-column returned carries an attribute \code{idx}, which is an \code{n}-by-2 matrix with every row the index of the corresponding entries of \code{x} and \code{y}, respectively.
-#' @details When using GEOS and not using s2, a spatial index is built on argument \code{x}; see \url{https://www.r-spatial.org/r/2017/06/22/spatial-index.html}. The reference for the STR tree algorithm is: Leutenegger, Scott T., Mario A. Lopez, and Jeffrey Edgington. "STR: A simple and efficient algorithm for R-tree packing." Data Engineering, 1997. Proceedings. 13th international conference on. IEEE, 1997. For the pdf, search Google Scholar.
+#' @details When using GEOS and not using s2, a spatial index is built on argument \code{x}; see \url{https://r-spatial.org/r/2017/06/22/spatial-index.html}. The reference for the STR tree algorithm is: Leutenegger, Scott T., Mario A. Lopez, and Jeffrey Edgington. "STR: A simple and efficient algorithm for R-tree packing." Data Engineering, 1997. Proceedings. 13th international conference on. IEEE, 1997. For the pdf, search Google Scholar.
 #' @seealso \link{st_union} for the union of simple features collections; \link{intersect} and \link{setdiff} for the base R set operations.
 #' @export
 #' @note To find whether pairs of simple feature geometries intersect, use
 #' the function \code{\link{st_intersects}} instead of \code{st_intersection}.
+#'
+#' When using GEOS and not using s2 polygons contain their boundary. When using s2 this is determined by the \code{model} defaults of \link[s2]{s2_options}, which can be overriden via the ... argument, e.g. \code{model = "closed"} to force DE-9IM compliant behaviour of polygons (and reproduce GEOS results).
 #' @examples
 #' set.seed(131)
 #' library(sf)
@@ -694,7 +878,7 @@ st_intersection.sf = function(x, y, ...) {
 		x[[ sf_column ]] = structure(geom, idx = NULL)
 		st_sf(x)
 	} else
-		geos_op2_df(x, y, geos_op2_geom("intersection", x, y))
+		geos_op2_df(x, y, geos_op2_geom("intersection", x, y, ...))
 }
 
 #' @name geos_binary_ops
@@ -736,7 +920,7 @@ st_difference.sf = function(x, y, ...) {
 		x[[ sf_column ]] = structure(geom, idx = NULL)
 		st_sf(x)
 	} else
-		geos_op2_df(x, y, geos_op2_geom("difference", x, y))
+		geos_op2_df(x, y, geos_op2_geom("difference", x, y, ...))
 }
 
 #' @name geos_binary_ops
@@ -782,8 +966,8 @@ st_snap.sfg = function(x, y, tolerance)
 st_snap.sfc = function(x, y, tolerance) {
 	if (isTRUE(st_is_longlat(x)))
 		stop("st_snap for longitude/latitude data not supported; use st_transform first?")
-	else if (inherits(tolerance, "units") && !is.na(st_crs(x)))
-		units(tolerance) = crs_parameters(st_crs(x))$ud_unit # coordinate units
+	else if (inherits(tolerance, "units") && !is.null(st_crs(x)$ud_unit))
+		units(tolerance) = st_crs(x)$ud_unit # coordinate units
 	tolerance = rep(tolerance, length.out = length(x))
 	st_sfc(CPL_geos_snap(st_geometry(x), st_geometry(y), as.double(tolerance)))
 }
@@ -794,14 +978,14 @@ st_snap.sf = function(x, y, tolerance)
 
 #' @name geos_combine
 #' @export
-#' @param by_feature logical; if TRUE, union each feature, if FALSE return a single feature that is the geometric union of the set of features
+#' @param by_feature logical; if TRUE, union each feature if \code{y} is missing or else each pair of features; if FALSE return a single feature that is the geometric union of the set of features in \code{x} if \code{y} is missing, or else the unions of each of the elements of the Cartesian product of both sets
 #' @param is_coverage logical; if TRUE, use an optimized algorithm for features that form a polygonal coverage (have no overlaps)
 #' @param y object of class \code{sf}, \code{sfc} or \code{sfg} (optional)
 #' @param ... ignored
 #' @seealso \link{st_intersection}, \link{st_difference}, \link{st_sym_difference}
-#' @return If \code{y} is missing, \code{st_union(x)} returns a single geometry with resolved boundaries, else the geometries for all unioned pairs of x[i] and y[j].
+#' @return If \code{y} is missing, \code{st_union(x)} returns a single geometry with resolved boundaries, else the geometries for all unioned pairs of `x[i]` and `y[j]`.
 #' @details
-#' If \code{st_union} is called with a single argument, \code{x}, (with \code{y} missing) and \code{by_feature} is \code{FALSE} all geometries are unioned together and an \code{sfg} or single-geometry \code{sfc} object is returned. If \code{by_feature} is \code{TRUE} each feature geometry is unioned. This can for instance be used to resolve internal boundaries after polygons were combined using \code{st_combine}. If \code{y} is provided, all elements of \code{x} and \code{y} are unioned, pairwise (and \code{by_feature} is ignored). The former corresponds to \link[rgeos:topo-bin-gUnion]{gUnaryUnion}, the latter to \link[rgeos:topo-bin-gUnion]{gUnion}.
+#' If \code{st_union} is called with a single argument, \code{x}, (with \code{y} missing) and \code{by_feature} is \code{FALSE} all geometries are unioned together and an \code{sfg} or single-geometry \code{sfc} object is returned. If \code{by_feature} is \code{TRUE} each feature geometry is unioned individually. This can for instance be used to resolve internal boundaries after polygons were combined using \code{st_combine}. If \code{y} is provided, all elements of \code{x} and \code{y} are unioned, pairwise if \code{by_feature} is TRUE, or else as the Cartesian product of both sets. 
 #'
 #' Unioning a set of overlapping polygons has the effect of merging the areas (i.e. the same effect as iteratively unioning all individual polygons together). Unioning a set of LineStrings has the effect of fully noding and dissolving the input linework. In this context "fully noded" means that there will be a node or endpoint in the output for every endpoint or line segment crossing in the input. "Dissolved" means that any duplicate (e.g. coincident) line segments or portions of line segments will be reduced to a single line segment in the output.	Unioning a set of Points has the effect of merging all identical points (producing a set with no duplicates).
 #' @examples
@@ -819,17 +1003,47 @@ st_union.sfg = function(x, y, ..., by_feature = FALSE, is_coverage = FALSE) {
 
 #' @export
 st_union.sfc = function(x, y, ..., by_feature = FALSE, is_coverage = FALSE) {
+	ll = isTRUE(st_is_longlat(x))
 	if (missing(y)) { # unary union, possibly by_feature:
-		ll = isTRUE(st_is_longlat(x))
-		if (ll && sf_use_s2() && !by_feature)
-			st_as_sfc(s2::s2_union_agg(x, ...), crs = st_crs(x))
-		else {
+		if (ll && sf_use_s2()) { 
+			if (! by_feature) { # see https://github.com/r-spatial/s2/issues/97 :
+				if (is_coverage)
+					st_as_sfc(s2::s2_coverage_union_agg(x, ...), crs = st_crs(x))
+				else
+					st_as_sfc(s2::s2_union_agg(x, ...), crs = st_crs(x)) 
+			} else
+				st_as_sfc(s2::s2_union(x, ...), crs = st_crs(x)) 
+		} else {
 			if (ll)
 				message_longlat("st_union")
-			st_sfc(CPL_geos_union(st_geometry(x), by_feature, is_coverage))
+			st_sfc(CPL_geos_union(x, by_feature, is_coverage))
 		}
-	} else
-		geos_op2_geom("union", x, y)
+	} else {
+		y = st_geometry(y)
+		stopifnot(st_crs(x) == st_crs(y))
+		if (by_feature) {
+			stopifnot(length(x) == length(y))
+			if (ll && sf_use_s2())
+				st_as_sfc(s2::s2_union(x, y, ...), crs = st_crs(x)) 
+			else {
+				if (ll)
+					message_longlat("st_union")
+				st_as_sfc(mapply(st_union, x, y, MoreArgs = list(is_coverage = is_coverage), SIMPLIFY = FALSE),
+						  crs = st_crs(x), precision = st_precision(x))
+			}
+		} else {
+			if (ll && sf_use_s2()) {
+				i = rep(seq_along(x), each = length(y))
+				j = rep(seq_along(y), length(x))
+				st_as_sfc(s2::s2_union(x[i], y[j], ...), crs = st_crs(x),
+					precision = st_precision(x)) 
+			} else {
+				if (ll)
+					message_longlat("st_union")
+				geos_op2_geom("union", x, y, ...)
+			}
+		}
+	}
 }
 
 #' @export
@@ -840,8 +1054,13 @@ st_union.sf = function(x, y, ..., by_feature = FALSE, is_coverage = FALSE) {
 			st_set_geometry(x, geom)
 		else
 			geom
-	} else
-		geos_op2_df(x, y, geos_op2_geom("union", x, y))
+	} else {
+		if (by_feature) {
+			df = cbind(st_drop_geometry(x), st_drop_geometry(y))
+			st_set_geometry(df, st_union(st_geometry(x), st_geometry(y), is_coverage = is_coverage))
+		} else
+			geos_op2_df(x, y, geos_op2_geom("union", x, y, ...))
+	}
 }
 
 #' Sample points on a linear geometry
@@ -871,8 +1090,8 @@ st_line_sample = function(x, n, density, type = "regular", sample = NULL) {
 	l = st_length(x)
 	distList = if (is.null(sample)) {
 		n = if (missing(n)) {
-			if (!is.na(st_crs(x)) && inherits(density, "units"))
-				units(density) = 1/crs_parameters(st_crs(x))$ud_unit # coordinate units
+			if (!is.null(st_crs(x)$ud_unit) && inherits(density, "units"))
+				units(density) = 1/st_crs(x)$ud_unit # coordinate units
 			round(rep(density, length.out = length(l)) * l)
 		} else
 			rep(n, length.out = length(l))
@@ -892,7 +1111,7 @@ st_line_sample = function(x, n, density, type = "regular", sample = NULL) {
 }
 
 #' Internal functions
-#' @name internal
+#' @keywords internal
 #' @param msg error message
 #' @export
 .stop_geos = function(msg) { #nocov start
