@@ -27,7 +27,7 @@ st_sample = function(x, size, ...) UseMethod("st_sample")
 #' when \code{type = "random"}.
 #' @param progress logical; if \code{TRUE} show progress bar (only if \code{size} is a vector).
 #' @return an \code{sfc} object containing the sampled \code{POINT} geometries
-#' @details if \code{x} has dimension 2 (polygons) and geographical coordinates (long/lat), uniform random sampling on the sphere is applied, see e.g. \url{http://mathworld.wolfram.com/SpherePointPicking.html}. 
+#' @details if \code{x} has dimension 2 (polygons) and geographical coordinates (long/lat), uniform random sampling on the sphere is applied, see e.g. \url{https://mathworld.wolfram.com/SpherePointPicking.html}. 
 #'
 #' For \code{regular} or \code{hexagonal} sampling of polygons, the resulting size is only an approximation.
 #'
@@ -35,6 +35,8 @@ st_sample = function(x, size, ...) UseMethod("st_sample")
 #' 
 #' Fibonacci sampling see: Alvaro Gonzalez, 2010. Measurement of Areas on a Sphere Using Fibonacci and Latitude-Longitude Lattices. 
 #' Mathematical Geosciences 42(1), p. 49-64
+#' 
+#' For regular sampling on the sphere, see also \code{geosphere::regularCoordinates}.
 #'
 #' Sampling methods from package \code{spatstat} are interfaced (see examples), and need their own parameters to be set. 
 #' For instance, to use \code{spatstat.random::rThomas()}, set \code{type = "Thomas"}.
@@ -50,13 +52,13 @@ st_sample = function(x, size, ...) UseMethod("st_sample")
 #' if (sf_extSoftVersion()["proj.4"] >= "4.9.0")
 #'   plot(p <- st_sample(x, 1000), add = TRUE)
 #' if (require(lwgeom, quietly = TRUE)) { # for st_segmentize()
-#' x2 = st_transform(st_segmentize(x, 1e4), st_crs("+proj=ortho +lat_0=30 +lon_0=45"))
-#' g = st_transform(st_graticule(), st_crs("+proj=ortho +lat_0=30 +lon_0=45"))
-#' plot(x2, graticule = g)
-#' if (sf_extSoftVersion()["proj.4"] >= "4.9.0") {
-#'   p2 = st_transform(p, st_crs("+proj=ortho +lat_0=30 +lon_0=45"))
-#'   plot(p2, add = TRUE)
-#' }
+#'   x2 = st_transform(st_segmentize(x, 1e4), st_crs("+proj=ortho +lat_0=30 +lon_0=45"))
+#'   g = st_transform(st_graticule(), st_crs("+proj=ortho +lat_0=30 +lon_0=45"))
+#'   plot(x2, graticule = g)
+#'   if (sf_extSoftVersion()["proj.4"] >= "4.9.0") {
+#'     p2 = st_transform(p, st_crs("+proj=ortho +lat_0=30 +lon_0=45"))
+#'     plot(p2, add = TRUE)
+#'   }
 #' }
 #' x = st_sfc(st_polygon(list(rbind(c(0,0),c(90,0),c(90,10),c(0,90),c(0,0))))) # NOT long/lat:
 #' plot(x)
@@ -88,9 +90,9 @@ st_sample = function(x, size, ...) UseMethod("st_sample")
 #' plot(st_sample(ls, 80))
 #' # spatstat example:
 #' if (require(spatstat.random)) {
-#'  x <- sf::st_sfc(sf::st_polygon(list(rbind(c(0, 0), c(10, 0), c(10, 10), c(0, 0)))))
-#'  # for spatstat.random::rThomas(), set type = "Thomas":
-#'  pts <- st_sample(x, kappa = 1, mu = 10, scale = 0.1, type = "Thomas") 
+#'   x <- sf::st_sfc(sf::st_polygon(list(rbind(c(0, 0), c(10, 0), c(10, 10), c(0, 0)))))
+#'   # for spatstat.random::rThomas(), set type = "Thomas":
+#'   pts <- st_sample(x, kappa = 1, mu = 10, scale = 0.1, type = "Thomas") 
 #' }
 #' @export
 #' @name st_sample
@@ -108,16 +110,16 @@ st_sample.sfc = function(x, size, ..., type = "random", exact = TRUE, warn_if_no
 		ret = if (progress) {
 				if (!requireNamespace("pbapply", quietly = TRUE))
 					stop("package pbapply required, please install it first")
-				pbapply::pblapply(1:length(x), function(i) st_sample(x[i], size[i], type = type, exact = exact, ...))
+				pbapply::pblapply(seq_along(x), function(i) st_sample(x[i], size[i], type = type, exact = exact, ...))
 			} else
-				lapply(1:length(x), function(i) st_sample(x[i], size[i], type = type, exact = exact, ...))
+				lapply(seq_along(x), function(i) st_sample(x[i], size[i], type = type, exact = exact, ...))
 		st_set_crs(do.call(c, ret), st_crs(x))
 	} else {
 		res = switch(max(st_dimension(x)) + 1,
 					 st_multipoints_sample(do.call(c, x), size = size, ..., type = type),
 					 st_ll_sample(st_cast(x, "LINESTRING"), size = size, ..., type = type),
 					 st_poly_sample(x, size = size, ..., type = type, by_polygon = by_polygon))
-		if (exact & type == "random" & all(st_geometry_type(res) == "POINT")) {
+		if (exact && type == "random" && all(st_geometry_type(res) == "POINT")) {
 			diff = size - length(res)
 			if (diff > 0) { # too few points
 				res_additional = st_sample_exact(x = x, size = diff, ..., 
@@ -137,6 +139,31 @@ st_sample.sfg = function(x, size, ...) {
 	st_sample(st_geometry(x), size, ...)
 }
 
+#' @export
+#' @name st_sample
+#' @param great_circles logical; if `TRUE`, great circle arcs are used to connect the bounding box vertices, if `FALSE` parallels (graticules)
+#' @param segments units, or numeric (degrees); segment sizes for segmenting a bounding box polygon if `great_circles` is `FALSE`
+#' @examples
+#' bbox = st_bbox(
+#'	c(xmin = 0, xmax = 40, ymax = 70, ymin = 60),
+#' 	crs = st_crs('OGC:CRS84')
+#' )
+#' set.seed(13531)
+#' s1 = st_sample(bbox, 400)
+#' st_bbox(s1) # within bbox
+#' s2 = st_sample(bbox, 400, great_circles = TRUE)
+#' st_bbox(s2) # outside bbox
+st_sample.bbox = function(x, size, ..., great_circles = FALSE, segments = units::set_units(2, "degree", mode = "standard")) {
+	polygon = st_as_sfc(x)
+	crs = st_crs(x)
+	if (isTRUE(st_is_longlat(x)) && !great_circles) {
+		st_crs(polygon) = NA_crs_ # to fool segmentize that we're on R2:
+		segments = units::drop_units(units::set_units(segments, "degree", mode = "standard"))
+		polygon = st_set_crs(st_segmentize(polygon, segments), crs)
+	}
+	st_sample(polygon, size, ...)
+}
+
 st_poly_sample = function(x, size, ..., type = "random",
                           offset = st_sample(st_as_sfc(st_bbox(x)), 1)[[1]],
 						  by_polygon = FALSE) {
@@ -158,19 +185,42 @@ st_poly_sample = function(x, size, ..., type = "random",
 				stop("hexagonal sampling on geographic coordinates not supported; consider projecting first")
 		} else if (type == "Fibonacci")
 			stop("Fibonacci sampling requires geographic (longlat) coordinates")
-
-		a0 = as.numeric(st_area(st_make_grid(x, n = c(1,1))))
-		a1 = as.numeric(sum(st_area(x)))
-		# st_polygon(list(rbind(c(-180,-90),c(180,-90),c(180,90),c(-180,90),c(-180,-90))))
-		# for instance has 0 st_area
-		if (is.finite(a0) && is.finite(a1) && a0 > a0 * 0.0 && a1 > a1 * 0.0) {
-			r = round(size * a0 / a1)
-			size = if (r == 0)
-					rbinom(1, 1, size * a0 / a1)
-				else
-					r
-		}
+		
+		global = FALSE
 		bb = st_bbox(x)
+		if (isTRUE(st_is_longlat(x))) {
+			if (sf_use_s2()) { # if FALSE, the user wants the coord ranges to be the bbox
+				if (!requireNamespace("lwgeom", quietly = TRUE))
+					warning("coordinate ranges not computed along great circles; install package lwgeom to get rid of this warning")
+				else 
+					bb = st_bbox(st_segmentize(st_as_sfc(bb),
+							units::set_units(1, "degree", mode = "standard"))) # get coordinate range on S2
+			}
+			R = s2::s2_earth_radius_meters()
+			toRad = pi / 180
+			h1 = sin(bb["ymax"] * toRad)
+			h2 = sin(bb["ymin"] * toRad)
+			a0 = 2 * pi * R^2. * (h1 - h2) * (bb["xmax"] - bb["xmin"]) / 360.
+			a1 = as.numeric(sum(st_area(x)))
+			if (!is.finite(a1))
+				stop("One or more geometries have a non-finite area")
+			if (a1 < 1) { # global polygon FIXME: 
+				a1 = a0
+				global = TRUE
+			}
+			size = round(size * a0 / a1)
+		} else {
+			a0 = as.numeric(st_area(st_as_sfc(bb)))
+			a1 = as.numeric(sum(st_area(x)))
+			# we're sampling from a box, so n should be size_desired * a0 / a1
+			if (is.finite(a0) && is.finite(a1) && a0 > a0 * 0.0 && a1 > a1 * 0.0) { # FIXME: reqs can be removed, now we handle long/lat separately?
+				r = size * a0 / a1
+				size = if (round(r) == 0)
+						rbinom(1, 1, r)
+					else
+						round(r)
+			}
+		}
 
 		pts = if (type == "hexagonal") {
 				dx = sqrt(a0 / size / (sqrt(3)/2))
@@ -181,25 +231,29 @@ st_poly_sample = function(x, size, ..., type = "random",
 					(offset[2] - bb["ymin"]) %% dx) + bb[c("xmin", "ymin")]
 				n = c(round((bb["xmax"] - offset[1])/dx), round((bb["ymax"] - offset[2])/dx))
 				st_make_grid(x, cellsize = c(dx, dx), offset = offset, n = n, what = "corners")
-			} else if (type == "random") {
-				lon = runif(size, bb[1], bb[3])
-				lat = if (isTRUE(st_is_longlat(x))) { # sampling on the sphere:
-					toRad = pi/180
-					lat0 = (sin(bb[2] * toRad) + 1)/2
-					lat1 = (sin(bb[4] * toRad) + 1)/2
-					y = runif(size, lat0, lat1)
-					asin(2 * y - 1) / toRad # http://mathworld.wolfram.com/SpherePointPicking.html
-				} else
-					runif(size, bb[2], bb[4])
-				m = cbind(lon, lat)
-				colnames(m) = NULL
-				st_sfc(lapply(seq_len(nrow(m)), function(i) st_point(m[i,])), crs = st_crs(x))
-			} else if (type == "Fibonacci") {
-				bb = st_bbox(x)
-				m = fiboGrid(size %/% 2, bb[c("xmin", "xmax")], bb[c("ymin", "ymax")])
-				st_sfc(lapply(seq_len(nrow(m)), function(i) st_point(m[i,])), crs = st_crs(x))
+			} else { 
+				m = if (type == "random") {
+						lon = runif(size, bb[1], bb[3])
+						lat = if (isTRUE(st_is_longlat(x))) { # sampling on the sphere:
+							toRad = pi/180
+							lat0 = (sin(bb[2] * toRad) + 1)/2
+							lat1 = (sin(bb[4] * toRad) + 1)/2
+							y = runif(size, lat0, lat1)
+							asin(2 * y - 1) / toRad # http://mathworld.wolfram.com/SpherePointPicking.html
+						} else
+							runif(size, bb[2], bb[4])
+						structure(cbind(lon, lat), dimnames = NULL)
+					} else if (type == "Fibonacci")
+						fiboGrid(size %/% 2, bb[c("xmin", "xmax")], bb[c("ymin", "ymax")])
+					else
+						stop("unknown value for type")
+				# st_sfc(lapply(seq_len(nrow(m)), function(i) st_point(m[i,])), crs = st_crs(x))
+				st_as_sf(as.data.frame(m), coords = 1:2, crs = st_crs(x))[["geometry"]]
 			}
-		pts[x] # cut out x from bbox
+		if (global)
+			pts
+		else
+			pts[x] # cut out x from bbox
 	} else { # try to go into spatstat
 		if (!requireNamespace("spatstat.random", quietly = TRUE))
 			stop("package spatstat.random required, please install it (or the full spatstat package) first")
