@@ -45,10 +45,12 @@ st_as_sf.data.frame = function(x, ..., agr = NA_agr_, coords, wkt,
 			x$geometry = st_as_sfc(as.character(x[[wkt]]))
 	} else if (! missing(coords)) {
 		cc = as.data.frame(lapply(x[coords], as.numeric))
-		if (na.fail && any(is.na(cc)))
+		if (na.fail && anyNA(cc))
 			stop("missing values in coordinates not allowed")
-		classdim = getClassDim(rep(0, length(coords)), length(coords), dim, "POINT")
-		x$geometry = structure( points_rcpp(as.matrix(cc), dim),
+		# classdim = getClassDim(rep(0, length(coords)), length(coords), dim, "POINT")
+		if (is.null(sf_column_name))
+			sf_column_name = "geometry"
+		x[[sf_column_name]] = structure( points_rcpp(as.matrix(cc), dim),
 			n_empty = 0L, precision = 0, crs = NA_crs_,
 			bbox = structure(
 				c(xmin = min(cc[[1]], na.rm = TRUE),
@@ -57,11 +59,12 @@ st_as_sf.data.frame = function(x, ..., agr = NA_agr_, coords, wkt,
 				ymax = max(cc[[2]], na.rm = TRUE)), class = "bbox"),
 			class =  c("sfc_POINT", "sfc" ), names = NULL)
 
-		if (is.character(coords))
-			coords = match(coords, names(x))
-
-		if (remove)
+		if (remove) {
+			if (is.character(coords))
+				coords = match(coords, names(x))
 			x = x[-coords]
+		}
+
 	}
 	st_sf(x, ..., agr = agr, sf_column_name = sf_column_name)
 }
@@ -192,8 +195,10 @@ list_column_to_sfc = function(x) {
 
 #' Create sf object
 #'
-#' Create sf, which extends data.frame-like objects with a simple feature list column
+#' Create sf, which extends data.frame-like objects with a simple feature list column.
+#' To convert a data frame object to `sf`, use [st_as_sf()]
 #' @name sf
+#' @aliases st_sf
 #' @param ... column elements to be binded into an \code{sf} object or a single \code{list} or \code{data.frame} with such columns; at least one of these columns shall be a geometry list-column of class \code{sfc} or be a list-column that can be converted into an \code{sfc} by \link{st_as_sfc}.
 #' @param crs coordinate reference system, something suitable as input to \link{st_crs}
 #' @param agr character vector; see details below.
@@ -265,7 +270,7 @@ st_sf = function(..., agr = NA_agr_, row.names,
 			x
 		else if (length(x) == 1) # ONLY one sfc
 			data.frame(row.names = row.names)
-		else if (!sfc_last & inherits(x, "data.frame"))
+		else if (!sfc_last && inherits(x, "data.frame"))
 			x
 		else if (sfc_last && inherits(x, "data.frame"))
 			x[-all_sfc_columns]
@@ -293,12 +298,18 @@ st_sf = function(..., agr = NA_agr_, row.names,
 	st_agr(df) = agr
 	if (! missing(crs))
 		st_crs(df) = crs
+
+	if (Sys.getenv("ADD_SF_NAMESPACE") == "true")
+		attr(df, ".sf_namespace") <- .sf_namespace
+
 	df
 }
 
+.sf_namespace <- function() NULL
+
 #' @name sf
 #' @param x object of class \code{sf}
-#' @param i record selection, see \link{[.data.frame}
+#' @param i record selection, see \link{[.data.frame}, or a \code{sf} object to work with the \code{op} argument
 #' @param j variable selection, see \link{[.data.frame}
 #' @param drop logical, default \code{FALSE}; if \code{TRUE} drop the geometry column and return a \code{data.frame}, else make the geometry sticky and return a \code{sf} object.
 #' @param op function; geometrical binary predicate function to apply when \code{i} is a simple feature object
@@ -368,6 +379,11 @@ st_sf = function(..., agr = NA_agr_, row.names,
 	}
 }
 
+#' @export
+"[<-.sf" = function(x, i, j, value) {
+	st_set_agr(NextMethod())
+}
+
 #' @export
 "[[<-.sf" = function(x, i, value) {
 	agr = st_agr(x)
@@ -392,7 +408,7 @@ st_sf = function(..., agr = NA_agr_, row.names,
 	x
 }
 
-#' @name sf
+#' @rdname sf
 #' @param n maximum number of features to print; can be set globally by \code{options(sf_max_print=...)}
 #' @export
 print.sf = function(x, ..., n = getOption("sf_max_print", default = 10)) {
@@ -401,7 +417,7 @@ print.sf = function(x, ..., n = getOption("sf_max_print", default = 10)) {
 	nf = length(x) - length(geoms)
 	app = paste("and", nf, ifelse(nf == 1, "field", "fields"))
 	if (any(!is.na(st_agr(x))))
-		app = paste0(app, "\n", "Attribute-geometry relationship: ", summarize_agr(x))
+		app = paste0(app, "\n", "Attribute-geometry relationship", ifelse(nf > 1, "s: ", ": "), summarize_agr(x))
 	if (length(geoms) > 1)
 		app = paste0(app, "\n", "Active geometry column: ", attr(x, "sf_column"))
 	print(st_geometry(x), n = 0, what = "Simple feature collection with", append = app)
@@ -450,6 +466,23 @@ as.data.frame.sf = function(x, ...) {
 	NextMethod()
 }
 
+#' @export
+duplicated.sf <- function(x, incomparables = FALSE, fromLast = FALSE, ...) {
+  if (length(x) != 1L) {
+    if (any(i <- vapply(x, is.factor, NA))) {
+      for (j in names(i[i])) {
+        x[[j]] <- lapply(x[[j]], as.numeric)
+      }
+    }
+    if (any(i <- (lengths(lapply(x, dim)) == 2L))) {
+      for (j in names(i[i])) {
+        x[[j]] <- lapply(x[[j]], split.data.frame, seq_len(nrow(x)))
+      }
+    }
+  }
+  NextMethod()
+}
+
 #' @export
 #' @name st_geometry
 #' @details if \code{x} is of class \code{sf}, \code{st_drop_geometry} drops the geometry of its argument, and reclasses it accordingly; otherwise it returns \code{x} unmodified. 
