@@ -25,6 +25,7 @@ chk_mpol = function(x) {
 sanity_check = function(x) {
 	d = st_dimension(x) # flags empty geoms as NA
 	if (any(d == 2, na.rm = TRUE)) { # the polygon stuff
+		x = st_cast(x[d == 2]) # convert GEOMETRY to POLYGON or MULTIPOLYGON, if possible
 		if (inherits(x, "sfc_POLYGON"))
 			st_sfc(lapply(x, chk_pol), crs = st_crs(x))
 		else if (inherits(x, "sfc_MULTIPOLYGON"))
@@ -35,6 +36,16 @@ sanity_check = function(x) {
 		x # nocov
 }
 
+#' @export
+#' @name st_transform
+#' @param src source crs
+#' @param dst destination crs
+#' @details \code{st_can_transform} returns a boolean indicating whether
+#' coordinates with CRS src can be transformed into CRS dst
+st_can_transform = function(src, dst) {
+	CPL_can_transform(st_crs(src), st_crs(dst))
+}
+
 #' Transform or convert coordinates of simple feature
 #'
 #' Transform or convert coordinates of simple feature
@@ -56,10 +67,13 @@ sanity_check = function(x) {
 #' @param check logical; if \code{TRUE}, perform a sanity check on resulting polygons
 #' @details Transforms coordinates of object to new projection. 
 #' Features that cannot be transformed are returned as empty geometries.
+#' Transforms using the \code{pipeline=} argument may fail if there is
+#' ambiguity in the axis order of the specified coordinate reference system;
+#' if you need the traditional GIS order, use \code{"OGC:CRS84"}, not
+#' \code{"EPSG:4326"}. Extra care is needed with the ESRI Shapefile format,
+#' because WKT1 does not store axis order unambigiously.
 #'
-#' @seealso Projecting simple feature geometries 
-#' to projections not supported by GDAL may be done by 
-#' \link[lwgeom]{st_transform_proj}, part of package lwgeom.
+#' @seealso \link[lwgeom]{st_transform_proj}, part of package lwgeom.
 #' 
 #' \link{sf_project} projects a matrix of coordinates, bypassing GDAL altogether
 #' @examples
@@ -162,20 +176,15 @@ st_wrap_dateline = function(x, options, quiet) UseMethod("st_wrap_dateline")
 #' @export
 #' @examples
 #' st_wrap_dateline(st_sfc(st_linestring(rbind(c(-179,0),c(179,0))), crs = 4326))
-#' library(maps)
-#' wrld <- st_as_sf(maps::map("world", fill = TRUE, plot = FALSE))
-#' wrld_wrap <- st_wrap_dateline(wrld, options = c("WRAPDATELINE=YES", "DATELINEOFFSET=180"),
-#'    quiet = TRUE)
-#' wrld_moll <- st_transform(wrld_wrap, "+proj=moll")
-#' plot(st_geometry(wrld_moll), col = "transparent")
-#' @details For a discussion of using \code{options}, see \url{https://github.com/r-spatial/sf/issues/280} and \url{https://github.com/r-spatial/sf/issues/541}
+#' @details For a discussion of using \code{options}, see \url{https://github.com/r-spatial/sf/issues/280} and \url{https://github.com/r-spatial/sf/issues/1983}
+#' @seealso \link{st_break_antimeridian}
 st_wrap_dateline.sfc = function(x, options = "WRAPDATELINE=YES", quiet = TRUE) {
 	if (is.na(st_crs(x)))
 		warning("crs not set: assuming geographic coordinates")
 	else
 		stopifnot(st_is_longlat(x))
 	stopifnot(is.character(options))
-	stopifnot(is.logical(quiet) && length(quiet) == 1)
+	stopifnot(is.logical(quiet), length(quiet) == 1)
 	st_sfc(CPL_wrap_dateline(x, options, quiet), crs = st_crs(x))
 }
 
