@@ -15,7 +15,6 @@ library(sf)
 library(DBI)
 library(RPostgres)
 library(testthat)
-context("sf: postgis using RPostgres")
 
 can_con <- function(x) inherits(x, "PqConnection")
 
@@ -57,6 +56,26 @@ test_that("can write to db", {
     expect_equal(nrow(DBI::dbReadTable(pg, "sf_meuse3__")), nrow(z) * 2)
     expect_silent(sf3 <- st_write(z, pg, "sf_meuse3__", delete_layer = TRUE))
     expect_true(st_crs(sf3) == st_crs(epsg_31370))
+    # also test write_sf options
+    expect_error(write_sf(pts, pg, "sf_meuse__", append = FALSE, delete_layer = FALSE), "exists")
+    expect_silent(write_sf(pts, pg, "sf_meuse__", delete_layer = TRUE))
+    expect_error(write_sf(pts, pg, "sf_meuse__", append = FALSE, delete_layer = FALSE))
+    expect_error(write_sf(pts[1, ], pg, "sf_meuse__", append = TRUE, delete_layer = TRUE))
+    expect_error(write_sf(pts, pg, "sf_meuse__", overwrite = TRUE), "deprecated")
+    expect_silent(write_sf(pts[1, ], pg, "sf_meuse__", append = TRUE))
+
+    p2 <- st_read(pg, "sf_meuse__")
+    expect_equal(nrow(p2), nrow(pts) + 1)
+    expect_silent(write_sf(pts, pg, "sf_meuse__", delete_layer = TRUE))
+})
+
+test_that("can create a missing table even if append is TRUE (#2206)", {
+	skip_if_not(can_con(pg), "could not connect to postgis database")
+	x <- st_sf(geometry = st_sfc(st_point(1:2)))
+	dbWriteTable(pg, "x", x, append = TRUE, temporary = TRUE)
+	col_type <- dbGetQuery(pg, "SELECT pg_typeof(geometry) as col_type FROM x")
+	expect_equal(unclass(col_type[["col_type"]]), "geometry")
+	dbExecute(pg, "drop table if exists x")
 })
 
 test_that("can handle multiple geom columns", {
@@ -88,7 +107,7 @@ test_that("sf can write units to database (#264)", {
     units(ptsu[["u"]]) <- units::as_units("km")
     expect_silent(st_write(ptsu, pg, "sf_units__", delete_layer = TRUE))
     r <- st_read(pg, "sf_units__")
-    expect_is(r[["u"]], "numeric")
+    expect_type(r[["u"]], "double")
     expect_equal(sort(r[["u"]]), sort(as.numeric(ptsu[["u"]])))
     dbRemoveTable(pg, "sf_units__")
 })
@@ -102,9 +121,9 @@ test_that("sf can read non-sf tables with geometries", {
 
 test_that("returns an `sf` object (#1039)", {
     skip_if_not(can_con(pg), "could not connect to postgis database")
-    expect_is(st_read(pg, query = "SELECT 'POINT(1 1)'::geometry"), "sf")
-    expect_is(st_read(pg, query = "SELECT 'POINT(1 1)'::geometry", as_tibble = TRUE), "sf")
-    expect_is(read_sf(pg, query = "SELECT 'POINT(1 1)'::geometry"), "sf")
+    expect_s3_class(st_read(pg, query = "SELECT 'POINT(1 1)'::geometry"), "sf")
+    expect_s3_class(st_read(pg, query = "SELECT 'POINT(1 1)'::geometry", as_tibble = TRUE), "sf")
+    expect_s3_class(read_sf(pg, query = "SELECT 'POINT(1 1)'::geometry"), "sf")
 })
 
 test_that("validates arguments", {
@@ -166,6 +185,8 @@ test_that("sf can preserve types (#592)", {
     st_write(dtypes, pg, delete_layer = TRUE)
     x <- st_read(pg, "dtypes")
     dtypes$fact <- as.character(dtypes$fact)
+    # R 4.1; EP:
+    attr(x$time, "tzone") = attr(dtypes$time, "tzone") = NULL
     expect_equal(x, dtypes)
     DBI::dbRemoveTable(pg, "dtypes")
 })
@@ -348,9 +369,9 @@ test_that("can read using driver", {
 test_that("Can override local crs", {
 	skip_if_not(can_con(pg), "could not connect to postgis database")
 	ewkb <- c(
-		wgs84 =       db_binary(st_set_crs(st_sfc(st_point(1:2)), 4326)),
-		unavailable = db_binary(st_set_crs(st_sfc(st_point(1:2)), make_empty_crs(1111))),
-		missing =     db_binary(st_sfc(st_point(1:2)))
+		wgs84 =       sf:::db_binary(st_set_crs(st_sfc(st_point(1:2)), 4326)),
+		unavailable = sf:::db_binary(st_set_crs(st_sfc(st_point(1:2)), sf:::make_empty_crs(1111))),
+		missing =     sf:::db_binary(st_sfc(st_point(1:2)))
 	)
 
 	queries <- paste0("select st_srid('", ewkb, "'::geometry) as srid")
@@ -364,43 +385,43 @@ test_that("Can safely manipulate crs", {
     skip_if_not(can_con(pg), "could not connect to postgis database")
     srid <- 4326
     crs <- st_crs(srid)
-    expect_true(db_find_srid(pg, srid = srid) == st_crs(srid))
-    expect_true(db_find_srtext(pg, crs) == st_crs(srid))
-    expect_error(db_insert_crs(pg, st_crs(srid)), "already exists")
-    expect_warning(expect_true(is.na(st_crs(get_new_postgis_srid(pg)))), "not found")
-    new_crs <- make_empty_crs(
-    	epsg = get_new_postgis_srid(pg),
+    expect_true(sf:::db_find_srid(pg, srid = srid) == st_crs(srid))
+    expect_true(sf:::db_find_srtext(pg, crs) == st_crs(srid))
+    expect_error(sf:::db_insert_crs(pg, st_crs(srid)), "already exists")
+    expect_warning(expect_true(is.na(st_crs(sf:::get_new_postgis_srid(pg)))), "not found")
+    new_crs <- sf:::make_empty_crs(
+    	epsg = sf:::get_new_postgis_srid(pg),
     	text = "+proj=longlat +datum=WGS84 +no_defs"
     )
-    expect_message(db_insert_crs(pg, new_crs, auth_name = "sf_test"), "Inserted local crs")
-    expect_error(db_insert_crs(pg, new_crs), "duplicate key")
-    expect_equal(delete_postgis_crs(pg, new_crs), 1)
-    expect_equal(delete_postgis_crs(pg, new_crs), 0)
+    expect_message(sf:::db_insert_crs(pg, new_crs, auth_name = "sf_test"), "Inserted local crs")
+    expect_error(sf:::db_insert_crs(pg, new_crs), "duplicate key")
+    expect_equal(sf:::delete_postgis_crs(pg, new_crs), 1)
+    expect_equal(sf:::delete_postgis_crs(pg, new_crs), 0)
 
     # set and delete
-    new_crs <- make_empty_crs(
+    new_crs <- sf:::make_empty_crs(
     	epsg = NA,
     	text = st_as_text(st_crs(epsg_31370))
     )
-    expect_message(new_srid <- db_insert_crs(pg, new_crs), "Inserted local crs")
-    expect_error(delete_postgis_crs(pg, new_crs), "Missing SRID")
+    expect_message(new_srid <- sf:::db_insert_crs(pg, new_crs), "Inserted local crs")
+    expect_error(sf:::delete_postgis_crs(pg, new_crs), "Missing SRID")
 
-    crs2 <- make_empty_crs(epsg(new_srid), st_as_text(st_crs(3857)))
-    expect_equal(delete_postgis_crs(pg, crs2), 0)  # crs doesn't match any crs
-    expect_equal(delete_postgis_crs(pg, new_srid), 1)
+    crs2 <- sf:::make_empty_crs(sf:::epsg(new_srid), st_as_text(st_crs(3857)))
+    expect_equal(sf:::delete_postgis_crs(pg, crs2), 0)  # crs doesn't match any crs
+    expect_equal(sf:::delete_postgis_crs(pg, new_srid), 1)
 
     # udpate
-    expect_message(db_insert_crs(pg, new_srid), "Inserted local crs")
+    expect_message(sf:::db_insert_crs(pg, new_srid), "Inserted local crs")
     new_srid[["wkt"]] <- crs2[["wkt"]]
-    expect_error(db_insert_crs(pg, new_srid), "already exists")
-    expect_message(db_insert_crs(pg, new_srid, update = TRUE), "Inserted local crs")
+    expect_error(sf:::db_insert_crs(pg, new_srid), "already exists")
+    expect_message(sf:::db_insert_crs(pg, new_srid, update = TRUE), "Inserted local crs")
 })
 
 
 test_that("new SRIDs are handled correctly", {
 	skip_if_not(can_con(pg), "could not connect to postgis database")
 
-	crs <- make_empty_crs(
+	crs <- sf:::make_empty_crs(
 		epsg = NA,
 		wkt = st_as_text(st_crs(epsg_31370))
 	)
@@ -423,7 +444,7 @@ test_that("schema_table", {
 
 test_that("Can find a crs", {
     skip_if_not(can_con(pg), "could not connect to postgis database")
-    expect_equal(db_find_srid(pg, st_crs(NA)), st_crs(NA))
+    expect_equal(sf:::db_find_srid(pg, st_crs(NA)), st_crs(NA))
     expect_error(sf:::delete_postgis_crs(pg, st_crs(NA)), "M|missing (crs)|(SRID)")
 })
 
